import datetime

import pytest

from lingo.invoicing.models import (
    Campaign,
    Counter,
    DraftInvoice,
    DraftInvoiceLine,
    Invoice,
    InvoiceLine,
    Pool,
    Regie,
)

pytestmark = pytest.mark.django_db


@pytest.mark.parametrize('draft', [True, False])
def test_invoice_total_amount(draft):
    regie = Regie.objects.create()
    invoice_model = DraftInvoice if draft else Invoice
    line_model = DraftInvoiceLine if draft else InvoiceLine

    campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_issue=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    pool = Pool.objects.create(
        campaign=campaign,
        draft=draft,
    )

    invoice = invoice_model.objects.create(
        date_publication=campaign.date_publication,
        date_payment_deadline=campaign.date_payment_deadline,
        date_issue=campaign.date_issue,
        regie=regie,
        pool=pool,
    )
    assert invoice.total_amount == 0
    invoice2 = invoice_model.objects.create(
        date_publication=campaign.date_publication,
        date_payment_deadline=campaign.date_payment_deadline,
        date_issue=campaign.date_issue,
        regie=regie,
        pool=pool,
    )
    assert invoice2.total_amount == 0

    # line with error status, ignored
    line = line_model.objects.create(
        event_date=datetime.date.today(),
        invoice=invoice,  # with invoice
        quantity=0,
        unit_amount=0,
        total_amount=0,
        status='error',
        pool=pool,
    )
    invoice.refresh_from_db()
    assert invoice.total_amount == 0
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # update line
    line.unit_amount = 10
    line.quantity = 1
    line.total_amount = 10
    line.save()
    invoice.refresh_from_db()
    # still in error
    assert invoice.total_amount == 0
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # update line status
    line.status = 'success'
    line.save()
    invoice.refresh_from_db()
    assert invoice.total_amount == 10
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # update other field
    line.unit_amount = 12  # total amount is wrong
    line.save()
    invoice.refresh_from_db()
    assert invoice.total_amount == 10
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # update total_amount
    line.status = 'success'
    line.total_amount = 12
    line.save()
    invoice.refresh_from_db()
    assert invoice.total_amount == 12
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # create line with invoice, status success
    line2 = line_model.objects.create(
        event_date=datetime.date.today(),
        invoice=invoice,
        quantity=1,
        unit_amount=20,
        total_amount=20,
        status='success',
        pool=pool,
    )
    invoice.refresh_from_db()
    assert invoice.total_amount == 32
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # change invoice
    line2.invoice = invoice2
    line2.save()
    invoice.refresh_from_db()
    assert invoice.total_amount == 12
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 20

    # delete line
    line2.delete()
    invoice.refresh_from_db()
    assert invoice.total_amount == 12
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # create line without invoice, status success
    line3 = line_model.objects.create(
        event_date=datetime.date.today(),
        quantity=1,
        unit_amount=20,
        total_amount=20,
        status='success',
        pool=pool,
    )
    invoice.refresh_from_db()
    assert invoice.total_amount == 12
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # set invoice
    line3.invoice = invoice
    line3.save()
    invoice.refresh_from_db()
    assert invoice.total_amount == 32
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # reset invoice
    line3.invoice = None
    line3.save()
    invoice.refresh_from_db()
    assert invoice.total_amount == 12
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0
    # no changes
    line3.save()
    invoice.refresh_from_db()
    assert invoice.total_amount == 12
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0

    # delete line
    line3.delete()
    invoice.refresh_from_db()
    assert invoice.total_amount == 12
    invoice2.refresh_from_db()
    assert invoice2.total_amount == 0


def test_counter():
    regie1 = Regie.objects.create()
    regie2 = Regie.objects.create()

    assert Counter.get_count(regie=regie1, name='foo') == 1
    assert Counter.objects.count() == 1
    counter1 = Counter.objects.get(regie=regie1, name='foo')
    assert counter1.value == 1

    assert Counter.get_count(regie=regie1, name='foo') == 2
    counter1.refresh_from_db()
    assert counter1.value == 2

    assert Counter.get_count(regie=regie1, name='foo') == 3
    counter1.refresh_from_db()
    assert counter1.value == 3

    assert Counter.get_count(regie=regie2, name='foo') == 1
    assert Counter.objects.count() == 2
    counter1.refresh_from_db()
    assert counter1.value == 3
    counter2 = Counter.objects.get(regie=regie2, name='foo')
    assert counter2.value == 1

    assert Counter.get_count(regie=regie2, name='bar') == 1
    assert Counter.objects.count() == 3
    counter1.refresh_from_db()
    assert counter1.value == 3
    counter2.refresh_from_db()
    assert counter2.value == 1
    counter3 = Counter.objects.get(regie=regie2, name='bar')
    assert counter3.value == 1


def test_regie_counter_name():
    regie = Regie.objects.create()
    assert regie.counter_name == '{yy}'

    assert regie.get_counter_name(datetime.date(2023, 1, 1)) == '23'
    assert regie.get_counter_name(datetime.date(2024, 1, 1)) == '24'

    regie.counter_name = '{yyyy}'
    regie.save()
    assert regie.get_counter_name(datetime.date(2023, 1, 1)) == '2023'
    assert regie.get_counter_name(datetime.date(2024, 1, 1)) == '2024'

    regie.counter_name = '{yy}-{mm}'
    regie.save()
    assert regie.get_counter_name(datetime.date(2023, 1, 1)) == '23-01'
    assert regie.get_counter_name(datetime.date(2023, 2, 1)) == '23-02'
    assert regie.get_counter_name(datetime.date(2024, 12, 1)) == '24-12'


def test_regie_format_number():
    regie = Regie.objects.create()
    assert regie.number_format == 'F{regie_id:02d}-{yy}-{mm}-{number:07d}'

    assert regie.format_number(datetime.date(2023, 2, 15), 42) == 'F%02d-23-02-0000042' % regie.pk
    assert regie.format_number(datetime.date(2024, 12, 15), 42000000) == 'F%02d-24-12-42000000' % regie.pk

    regie.number_format = 'Ffoobar-{yyyy}-{number:08d}'
    regie.save()
    assert regie.format_number(datetime.date(2023, 2, 15), 42) == 'Ffoobar-2023-00000042'
    assert regie.format_number(datetime.date(2024, 12, 15), 42000000) == 'Ffoobar-2024-42000000'
